use sqlx::SqlitePool;

use crate::types::uuid::Uuid;

use super::users::User;

pub struct Device {
    pub uuid: Uuid,
    pub user_uuid: Uuid,
    pub device_id: String,
    pub display_name: String,
}

impl Device {
    pub fn new(user: &User, device_id: &str, display_name: &str) -> anyhow::Result<Self> {
        Ok(Self {
            uuid: uuid::Uuid::new_v4().into(),
            user_uuid: user.uuid.clone(),
            device_id: device_id.to_owned(),
            display_name: display_name.to_owned(),
        })
    }

    pub async fn create(&self, conn: &SqlitePool) -> anyhow::Result<Self> {
        Ok(sqlx::query_as!(
            Self,
            "insert into devices(uuid, user_uuid, device_id, display_name)
            values(?, ?, ?, ?)
            returning uuid as 'uuid: Uuid', user_uuid as 'user_uuid: Uuid', device_id, display_name",
            self.uuid,
            self.user_uuid,
            self.device_id,
            self.display_name)
            .fetch_one(conn).await?
        )
    }

    pub async fn find_for_user(
        conn: &SqlitePool,
        user: &User,
        device_id: &str,
    ) -> anyhow::Result<Self> {
        let user_uuid = user.uuid.clone();
        Ok(sqlx::query_as!(
            Self,
            "select uuid as 'uuid: Uuid', user_uuid as 'user_uuid: Uuid', device_id, display_name from devices where user_uuid = ? and device_id = ?",
            user_uuid, device_id)
            .fetch_one(conn).await?
        )
    }

    pub async fn find_by_uuid(conn: &SqlitePool, uuid: &Uuid) -> anyhow::Result<Self> {
        Ok(sqlx::query_as!(
            Self,
            "select uuid as 'uuid: Uuid', user_uuid as 'user_uuid: Uuid', device_id, display_name from devices where uuid = ?",
            uuid)
            .fetch_one(conn).await?
        )
    }

    pub async fn user(&self, conn: &SqlitePool) -> anyhow::Result<User> {
        User::find_by_uuid(conn, &self.user_uuid).await
    }

    /// Get the device's id.
    #[must_use]
    pub fn uuid(&self) -> &Uuid {
        &self.uuid
    }
}
